# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import six
import time

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimCE
from driverng.astm import HprimList
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.hl7_sil")


class Hl7SilSegmentMSH(HeaderSegment):
    sending_application_code = HL7String((3, 1), default="MCA")
    sending_application_libelle = HL7String((3, 2))
    receiving_application_code = HL7String((5, 1), default="")
    receiving_application_libelle = HL7String((5, 2), default="")
    message_date = HL7Date(7)
    message_type = HL7String((9, 1), default="OUL")
    message_code = HL7String((9, 2), default="R22")
    message_name = HL7String((9, 3), default="OUL_R22")
    control_id = HL7String(10)
    version = HL7String(12, default="2.5.1")
    accept_acknowledgment_type = HL7String(15, default="AL")
    application_acknowlegment_type = HL7String(16, default="NE")
    character_set = HL7String(18, default="UNICODE UTF-8")

    patients = HL7Foreign("PID", "header")
    specimens = HL7Foreign("SPM", "header")


class Hl7SilSegmentPID(Segment):
    # IPP
    patient_id = HL7String(3)
    # Numéro admission clinique
    patient_identifier_list = HL7String(4)
    # IPP Hopital
    alternate_patient_id = HL7String(5)
    patient_name = HL7String((6, 1))
    patient_firstname = HL7String((6, 2))
    maiden_name = HL7String(7)
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    sex = HL7String(9)
    # 	LENGTH	DATA TYPE	OPTIONALITY	REPEATABILITY	TABLE
    # PID.11.1 - Street Address	0	SAD	O	-	Street	￼expand_more
    # PID.11.2 - Other Designation	0	ST	O	-		￼expand_more
    # PID.11.3 - City	0	ST	O	-	City	￼expand_more
    # PID.11.4 - State Or Province	0	ST	O	-	State	￼expand_more
    # PID.11.5 - Zip Or Postal Code	0	ST	O	-	ZipCode	￼expand_more
    # PID.11.6 - Country	3	ID	O	-	0399	￼expand_more
    # PID.11.7 - Address Type
    # 10, rue test^chemin tutu^Toulouse^^31000
    street_address = HL7String((12, 1))
    other_street_address = HL7String((12, 2))
    city_address = HL7String((12, 3))
    state_address = HL7String((12, 4))
    zip_address = HL7String((12, 5))
    country_address = HL7String((12, 6))
    phone_number = HL7String((15, 1))
    phone_suffix = HL7String((15, 2))
    patient_email = HL7String((15, 4))

    visits = HL7Foreign("PV1", "patient")
    specimens = HL7Foreign("SPM", "patient")
    infos = HL7Foreign("NTE", "patient")


class Hl7SilSegmentPV1(Segment):
    location = HL7String((4, 1))
    chambre = HL7String((4, 2))
    lit = HL7String((4, 3))
    admission_type = HL7String(5)

    # Prescripteur
    attending_doctor_code = HL7String((8, 1))
    attending_doctor_first_name = HL7String((8, 2))
    attending_doctor_name = HL7String((8, 3))
    attending_doctor_adeli = HL7String((8, 4))
    attending_doctor_finess = HL7String((8, 5))
    attending_doctor_rpps = HL7String((8, 6))
    attending_doctor_mail = HL7String((8, 7))

    # Médecin traitant
    referring_doctor_code = HL7String((9, 1))
    referring_doctor_firstname = HL7String((9, 2))
    referring_doctor_name = HL7String((9, 3))
    referring_doctor_adeli = HL7String((9, 4))
    referring_doctor_finess = HL7String((9, 5))
    referring_doctor_rpps = HL7String((9, 6))
    referring_doctor_mail = HL7String((9, 7))
    # Numéro d'hospitalisation
    visit_number = HL7String(20)
    # Date d'entrée
    admit_date = HL7Date(45)
    # Date de sortie
    discharge_date = HL7Date(46)

    orders = HL7Foreign("OBR", "patient")
    specimens = HL7Foreign("SPM", "patient")
    infos = HL7Foreign("NTE", "patient")


class Hl7SilSegmentPD1(Segment):
    patient_primary_facility = HL7String(4)


class Hl7SilSegmentSPM(Segment):
    # Nature id pour les patients
    # Nom du controle pour les QC
    specimen_id = HL7String(3)
    # Type de prélèvement
    specimen_type = HL7String(5)
    typ = HL7String(12, default="P")
    # Date et heure de prélèvement
    specimen_collection_date = HL7Date(18)
    # Date et heure de demande
    specimen_received_date = HL7Date(19)
    # Date et heure d'expiration du tube
    specimen_expiration_date = HL7Date(20)

    orders = HL7Foreign("OBR", "patient")
    infos_samples = HL7Foreign("SAC", "patient")
    common_orders = HL7Foreign("ORC", "patient")
    infos = HL7Foreign("NTE", "patient")


class Hl7SilSegmentEQU(Segment):
    container_identifier = HL7String(4)
    carrier_identifier = HL7String(11)
    position_carrier = HL7String(12)


class Hl7SilSegmentSAC(Segment):
    # Code barre pour les patients
    # Numéro du controle pour les QC
    container_identifier = HL7String(4)
    # Rack pour les patients
    # Numéro de lot pour les QC
    rack = HL7String(11)
    position = HL7String(12)

    orders = HL7Foreign("OBR", "patient")
    infos = HL7Foreign("NTE", "patient")


class Hl7SilSegmentORC(Segment):
    # NW : Nouveau
    # CA : Annulation
    # PR : Antériorité
    # XO : Ajout
    order_control = HL7String(2)
    # Numéro de demande
    placer_order_number = HL7String(3)
    # Code barre
    filler_order_number = HL7String(4)
    status = HL7String(5)
    request_date = HL7Date(16)
    # Laboratoire demandeur
    ordering_facility_name = HL7String(22)
    # Adresse du laboratoire demandeur
    ordering_facility_address = HL7String(23)
    # Téléphone du laboratoire demandeur
    ordering_facility_phone_number = HL7String(24)
    infos = HL7Foreign("NTE", "common_order")
    orders = HL7Foreign("OBR", "common_order")


class Hl7SilSegmentOBR(Segment):
    # Numéro de demande
    placer_order_number = HL7String(3)
    # Code barre ou nom du controle
    filler_order_number = HL7String(4)
    # Code du test
    test_code = HL7String((5, 1))
    # Libellé du test
    test_label = HL7String((5, 2))
    # Renseignement clinique
    relevant_clinical_info = HL7String(14)
    code_doctor = HL7String((17, 1))
    name_doctor = HL7String((17, 2))
    email_doctor = HL7String((17, 3))
    telephone_doctor = HL7String((18, 1))
    # service prescripteur
    ordering_service_code = HL7String((17, 22, 1))
    ordering_service_name = HL7String((17, 22, 2))
    # Collection date
    start_date = HL7Date((28, 4))
    # A/S/R
    priority = HL7String((28, 6))

    interpreter_code = HL7String((33, 1))
    interpreter_name = HL7String((33, 2))
    technician_code = HL7String((35, 1))
    technician_name = HL7String((35, 2))

    results = HL7Foreign("OBX", "orders")
    infos = HL7Foreign("NTE", "orders")


class Hl7SilSegmentOBX(Segment):
    value_type = HL7String(3)
    test_code = HL7String((4, 1))
    test_label = HL7String((4, 2))
    alternate_identifier = HL7String((4, 3))
    rank = HL7String((4, 4))
    sub_code = HL7String(5)
    observation_result = HprimList(6, basefield=HL7String(1))
    observation_result_code = HL7String((6, 1))
    observation_result_label = HL7String((6, 2))
    observation_result_codification = HL7String((6, 3))
    bacterio_results = HprimCE((6, 1))
    sensitivity = HL7String((6, 2))
    mic = HL7String((6, 3))
    diameter = HL7String((6, 4))
    cutoff = HL7String((6, 5))
    id_results = HprimCE((6, 2))
    qty_results = HprimCE((6, 3))
    units = HL7String(7)
    abnormal_flags = HprimList(
        9, basefield=HL7String(1)
    )  # Indic Norm, Alarmes analyseurs
    result_status = HL7String(12)
    result_date_str = HL7String(15)
    result_date = HL7Date(15)
    germ_test_code = HL7String((16, 3))
    germ_rank = HL7String((16, 6))
    # Valideur
    observer = HL7String(17)
    instrument = HL7String((19, 1))
    instrument_id = HL7String((19, 2))

    dilution = HL7Foreign("TCD", "results")
    commentaires = HL7Foreign("NTE", "results")
    reactifs = HL7Foreign("INV", "results")


class Hl7SilSegmentTCD(Segment):
    test_code = HL7String((2, 1))
    dilution_factor = HL7String((3, 4))


class Hl7SilSegmentNTE(Segment):
    # L : MCA
    # P : SIL
    source_comment = HL7String(3)
    comment = HL7String(4)


class Hl7SilSegmentINV(Segment):
    # Code réactif
    reagent_code = HL7String(2)
    # Type de contenant
    substance_status = HL7String(3)
    # Type de réactif
    reagent_type = HL7String(4)
    # Numéro de série du lot du réactif
    reagent_serial_number = HL7String(5)
    # Date d'expiration du lot du réactif
    reagent_expiration_date = HL7Date(13)


class Driver(AstmMessageV2):
    SPEC = {"code": "hl7_sil", "label": "HL7 SIL"}
    ENCODING = "utf-8"

    SEGMENT_CLASS_MAP = {
        "MSH": Hl7SilSegmentMSH,
        "PID": Hl7SilSegmentPID,
        "PD1": Hl7SilSegmentPD1,
        "PV1": Hl7SilSegmentPV1,
        "SPM": Hl7SilSegmentSPM,
        "SAC": Hl7SilSegmentSAC,
        "ORC": Hl7SilSegmentORC,
        "OBR": Hl7SilSegmentOBR,
        "OBX": Hl7SilSegmentOBX,
        "TCD": Hl7SilSegmentTCD,
        "NTE": Hl7SilSegmentNTE,
        "INV": Hl7SilSegmentINV,
        "EQU": Hl7SilSegmentEQU,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option(
            "hl7_sil_sender_code", label="Code émetteur", default="MCA"
        ),
        AstmMessageV2.Option(
            "hl7_sil_sender_label", label="Libellé émetteur", default=""
        ),
        AstmMessageV2.Option(
            "hl7_sil_receiver_code", label="Code récepteur", default=""
        ),
        AstmMessageV2.Option(
            "hl7_sil_receiver_label", label="Libellé récepteur", default=""
        ),
        AstmMessageV2.BooleanOption(
            "option_envoyer_fic",
            label="Envoyer les pièces jointes de la demande dans des segments OBX",
            default=False,
        ),
        AstmMessageV2.BooleanOption(
            "option_position_demande_id",
            label="Lire le no de demande sur le champ PID.3.1 (Patient Identifier List) en priorité sur le champ OBR2.1 (Placer Order Numbe)",
            default=False,
        ),
        AstmMessageV2.BooleanOption(
            "option_alarme_analytique",
            label="Envoyer les alarmes analyseurs dans le segment OBX",
            default=False,
        ),
    ]

    def __init__(self, **kwargs):
        self.sender_code = kwargs.pop("hl7_sil_sender_code", "MCA")
        self.sender_label = kwargs.pop("hl7_sil_sender_label", "")
        self.receiver_code = kwargs.pop("hl7_sil_receiver_code", "")
        self.receiver_label = kwargs.pop("hl7_sil_receiver_label", "")
        self.option_envoyer_fic = kwargs.pop("option_envoyer_fic", False)
        self.option_position_demande_id = kwargs.pop(
            "option_position_demande_id", False
        )
        self.option_alarme_analytique = kwargs.pop("option_alarme_analytique", False)
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return Hl7SilSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        # Encoder un message de type résultat SIL
        if worklist.context != "result":
            raise NotImplementedError("context %s" % (worklist.context,))
        worklist_sample_types = list(set([s.type for s in worklist.samples]))
        qc = worklist_sample_types == ["control"]
        message = self.create(
            sending_application_code=self.sender_code,
            sending_application_libelle=self.sender_label,
            receiving_application_code=self.receiver_code,
            receiving_application_libelle=self.receiver_label,
            message_date=context["now"],
            control_id=context.get("control_id", int(time.time() * 1000)),
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", "A": "S", worklist.ELSE: "R"},
            action={"new": "NW", "add": "XO", "del": "CA"},
        ):
            if not qc:
                seg_pid = message.patients.add(
                    patient_id=patient.patient_id,
                    patient_identifier_list=patient.patient_id,
                    patient_name=patient.last_name,
                    patient_firstname=patient.first_name,
                    maiden_name=patient.maiden_name,
                    birthdate=patient.birthdate,
                    sex=patient.sex,
                )
                physician = [p for p in patient.physicians if p.type == "attending"]
                prescriber = [p for p in patient.physicians if p.type == "prescriber"]
                seg_pv1 = seg_pid.visits.add(
                    location=patient.location,
                    attending_doctor_code=physician[0].code if physician else "",
                    attending_doctor_last_name=physician[0].last_name
                    if physician
                    else "",
                    attending_doctor_first_name=physician[0].first_name
                    if physician
                    else "",
                    referring_doctor_code=prescriber[0].code if prescriber else "",
                    referring_doctor_last_name=prescriber[0].last_name
                    if prescriber
                    else "",
                    referring_doctor_first_name=prescriber[0].first_name
                    if prescriber
                    else "",
                    admit_date=patient.admission_date,
                    discharge_date=patient.admission_end_date,
                    visit_number=patient.alternate_patient_id,
                )
                for sample, action, priority in patient.samples:
                    seg_spm = seg_pv1.specimens.add(
                        specimen_id=sample.sample_id,
                        specimen_type=sample.nature["code"],
                        specimen_received_date=sample.request_date,
                        specimen_collection_date=sample.collection_date,
                    )
                    antibio_result = False
                    identification_result = False
                    for result in sample.results:
                        if isinstance(result.value, list):
                            values = result.value
                            for value in values:
                                if "antibiogram" in value:
                                    antibio_result = True
                                if "json" in value and "germes" in value["json"]:
                                    identification_result = True
                        if not antibio_result and not identification_result:
                            seg_obr = seg_spm.orders.add(
                                placer_order_number=sample.request_id,
                                filler_order_number=sample.sample_id,
                                test_code=result.code,
                                name_test=result.label,
                                relevant_clinical_info=sample.clinical_information,
                                code_doctor=prescriber[0].code if prescriber else "",
                                name_doctor=prescriber[0].last_name
                                if prescriber
                                else "",
                                phone_doctor=prescriber[0].phone if prescriber else "",
                                start_date=sample.collection_date,
                                priority=priority,
                                technician_code=result.operator,
                            )
                            if isinstance(result.value[0], dict):
                                observation_result = result.value[0].get("value")
                            else:
                                observation_result = result.value
                            seg_obx = seg_obr.results.add(
                                value_type=result.type,
                                test_code=result.code,
                                test_label=result.label,  # if result.type == "CE" else "",
                                test_codification="L" if result.type == "CE" else "",
                                observation_result_code=observation_result,
                                abnormal_flags=result.flags
                                if self.option_alarme_analytique
                                else result.indic_norm,
                                units=result.unit,
                                result_status="P",
                                result_date=result.date,
                                observer=result.validator,
                                instrument=result.instrument,
                            )
                            if result.dilution:
                                seg_obx.dilution.add(
                                    test_code=result.code,
                                    dilution_factor=seg_obx.dilution,
                                )
                            if isinstance(result.value, six.string_types):
                                values = [{"value": v} for v in result.value.split(";")]
                            elif isinstance(result.value[0], dict):
                                values = result.value[0]
                                seg_obx.observation_result_label = values["label"]
                                seg_obx.observation_result_codification = values[
                                    "codification"
                                ]
                                if result.type == "GC" and all(
                                    [not v.get("label") for v in result.value]
                                ):
                                    # Graphe à pas fixe de type y^y^y Dans
                                    # ce cas il faut renseigner les champs
                                    points = [str(v.get("value")) for v in result.value]
                                    seg_obx.observation_result = points
                                    # TODO(eda) ou mettre pas et origine
                                    # seg_obx.gc_pas = "1"
                                    # seg_r.obxc_origine = "0"

                            elif isinstance(result.value, list):
                                values = result.value
                            else:
                                raise NotImplementedError
                            for comment in result.comments:
                                if seg_obx is not None:
                                    seg_obx.commentaires.add(
                                        source_comment="L", comment=comment
                                    )
                    if antibio_result or identification_result:
                        seg_obr = seg_spm.orders.add(
                            placer_order_number=sample.request_id,
                            filler_order_number=sample.sample_id,
                            test_code=result.code,
                            name_test=result.label,
                            relevant_clinical_info=sample.clinical_information,
                            code_doctor=prescriber[0].code if prescriber else "",
                            name_doctor=prescriber[0].last_name if prescriber else "",
                            phone_doctor=prescriber[0].phone if prescriber else "",
                            start_date=sample.collection_date,
                            priority=priority or "R",
                            technician_code=result.operator,
                        )
                        if antibio_result:
                            for result in sample.results:
                                seg_obx = seg_obr.results.add(
                                    value_type=result.type,
                                    test_code=result.code,
                                    alternate_identifier="GERMEATB",
                                    rank=result.sub_code,
                                    result_status=result.status,
                                    result_date=result.date,
                                    observer=result.validator,
                                    instrument=result.instrument,
                                )
                                seg_obx.bacterio_results.code1 = result.value[0].get(
                                    "value"
                                )
                                seg_obx.bacterio_results.label1 = result.value[0].get(
                                    "label"
                                )
                                seg_obx.bacterio_results.codification1 = result.value[
                                    0
                                ].get("codification")
                                if isinstance(result.value, list):
                                    values = result.value
                                    rank = 0
                                    for value in values:
                                        if "antibiogram" in value:
                                            antibiograms = value.get("antibiogram")
                                            for antibiogram in antibiograms:
                                                rank += 1
                                                seg_obx = seg_obr.results.add(
                                                    value_type=result.type,
                                                    test_code=value.get("value"),
                                                    alternate_identifier="ANTIBIOTIC",
                                                    rank=rank,
                                                    sensitivity=antibiogram.get(
                                                        "sensitivity"
                                                    ),
                                                    mic=antibiogram.get("mic"),
                                                    diameter=antibiogram.get(
                                                        "diameter"
                                                    ),
                                                    cutoff=antibiogram.get("cutoff"),
                                                    result_status=result.status,
                                                    result_date=result.date,
                                                    observer=result.validator,
                                                    instrument=result.instrument,
                                                    germ_test_code=result.code,
                                                    germ_rank=result.sub_code,
                                                )
                                                seg_obx.bacterio_results.code1 = antibiogram.get(
                                                    "antibiotic"
                                                )
                                                seg_obx.bacterio_results.label1 = antibiogram.get(
                                                    "antibiotic_texte_code"
                                                )[
                                                    1
                                                ]
                                                seg_obx.bacterio_results.codification1 = antibiogram.get(
                                                    "antibiotic_texte_code"
                                                )[
                                                    2
                                                ]
                        if identification_result:
                            seg_obx = seg_obr.results.add(
                                value_type=result.type,
                                test_code=result.code,
                                sub_code=result.sub_code,
                                observation_result_code=result.value[0].get("value"),
                                observation_result_label=result.value[0].get("label"),
                                observation_result_codification=result.value[0].get(
                                    "codification"
                                ),
                                result_status=result.status,
                                result_date=result.date,
                                observer=result.validator,
                                instrument=result.instrument,
                            )
                            if isinstance(result.value, list):
                                values = result.value
                                for value in values:
                                    if "json" in value and "germes" in value["json"]:
                                        germes = value["json"].get("germes")
                                        rank = int(result.sub_code)
                                        ids = (
                                            value["schema"]
                                            .get("properties")
                                            .get("germes")
                                            .get("items")
                                            .get("properties")
                                            .get("id")
                                            .get("format")
                                            .get("textecodes")
                                        )
                                        qtys = (
                                            value["schema"]
                                            .get("properties")
                                            .get("germes")
                                            .get("items")
                                            .get("properties")
                                            .get("qty")
                                            .get("format")
                                            .get("textecodes")
                                        )
                                        for germe in germes:
                                            rank += 1
                                            seg_obx = seg_obr.results.add(
                                                value_type=result.type,
                                                test_code=result.code,
                                                alternate_identifier="COMPLEXE",
                                                rank=rank,
                                                result_status=result.status,
                                                observation_result_code=(
                                                    germes.index(germe)
                                                )
                                                + 1,
                                                result_date=result.date,
                                                observer=result.validator,
                                                instrument=result.instrument,
                                                germ_test_code=result.code,
                                            )
                                            seg_obx.id_results.code1 = germe.get("id")
                                            seg_obx.id_results.label1 = ids.get(
                                                germe.get("id")
                                            )[1]
                                            seg_obx.id_results.codification1 = "id"
                                            seg_obx.qty_results.code1 = germe.get("qty")
                                            seg_obx.qty_results.label1 = qtys.get(
                                                germe.get("qty")
                                            )[1]
                                            seg_obx.qty_results.codification1 = "qty"

            else:
                for sample, action, priority in patient.samples:
                    seg_spm = message.specimens.add(
                        # Nature id pour les patients
                        # Nom du controle pour les QC
                        specimen_id=sample.sample_id,
                        # Type de prélèvement
                        specimen_type=sample.nature["code"],
                        # Date et heure de prélèvement
                        specimen_collection_date=sample.collection_date,
                        specimen_received_date=sample.request_date,
                        typ="QC",
                        # Date et heure d'expiration du tube
                        # specimen_expiration_date = HL7String(20)
                    )
                    seg_spm.infos_samples.add(
                        # Code barre pour les patients
                        # Numéro du controle pour les QC
                        container_identifier=sample.sample_id,
                        # Rack pour les patients
                        # Numéro de lot pour les QC
                        rack=sample.rack,
                        # position = sample.position,
                    )
                    for result in sample.results:
                        seg_obr = seg_spm.orders.add(
                            placer_order_number=sample.request_id,
                            filler_order_number=sample.sample_id,
                            test_code=result.code,
                            name_test=result.code,
                            start_date=sample.collection_date,
                            technician_code=result.operator,
                        )
                        if isinstance(result.value[0], dict):
                            observation_result = result.value[0].get("value")
                        else:
                            observation_result = result.value
                        seg_obx = seg_obr.results.add(
                            value_type=result.type,
                            test_code=result.code,
                            test_label=result.label if result.type == "CE" else "",
                            test_codification="L" if result.type == "CE" else "",
                            observation_result_code=observation_result,
                            abnormal_flags=result.flags,
                            units=result.unit,
                            result_status="P",
                            result_date=result.date,
                            observer=result.validator,
                            instrument=result.instrument,
                        )
                        for lot in result.lots:
                            seg_obx.reactifs.add(
                                # Code réactif
                                reagent_code=lot.name,
                                # Type de contenant
                                # substance_status = '',
                                # Type de réactif
                                reagent_type=lot.type,
                                # Numéro de série du lot du réactif
                                reagent_serial_number=lot.lot,
                                # Date d'expiration du lot du réactif
                                reagent_expiration_date=lot.expiration,
                            )

        return message

    def decode_message(self, message):
        # Décoder une demande provenant du SIL
        worklist = Worklist(
            context="order",
            sender_id=message.sending_application_code,
            receiver_id=message.receiving_application_code,
        )
        for seg_pid in message.patients:
            location = None
            no_admission = None
            date_admission = None
            chambre = None
            lit = None
            if seg_pid.visits:
                if seg_pid.visits[0].location:
                    location = seg_pid.visits[0].location
                if seg_pid.visits[0].visit_number:
                    no_admission = seg_pid.visits[0].visit_number
                    date_admission = seg_pid.visits[0].admit_date
                    chambre = seg_pid.visits[0].chambre
                    lit = seg_pid.visits[0].lit
                patient_specimens = seg_pid.visits[0].specimens
            else:
                patient_specimens = seg_pid.specimens
            for seg_spm in patient_specimens:
                list_orders = [(order, None) for order in seg_spm.orders]  # Sans ORC
                for common_order in seg_spm.common_orders:
                    for seg_obr in common_order.orders:
                        list_orders.append((seg_obr, common_order))

                for seg_obr, common_order in list_orders:
                    action = "add"
                    if common_order and common_order.order_control == "CA":
                        action = "del"
                    request_id = None
                    if (
                        self.option_position_demande_id
                        and seg_pid.patient_identifier_list
                    ):
                        request_id = seg_pid.patient_identifier_list
                    elif seg_obr.placer_order_number:
                        request_id = seg_obr.placer_order_number
                    elif seg_pid.patient_identifier_list:
                        request_id = seg_pid.patient_identifier_list
                    else:
                        raise RuntimeError(
                            "L'identification de la demande est absente."
                        )
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_obr.filler_order_number,
                        request_id=request_id,
                        patient_id=seg_pid.patient_id,
                        last_name=seg_pid.patient_name,
                        first_name=seg_pid.patient_firstname,
                        maiden_name=seg_pid.maiden_name,
                        birthdate=seg_pid.birthdate,
                        sex=seg_pid.sex,
                        location=location,
                        alternate_patient_id=no_admission,
                        admission_date=date_admission,
                        room=chambre,
                        bed=lit,
                        collection_date=seg_spm.specimen_collection_date
                        or seg_obr.start_date,
                        request_date=(
                            (common_order and common_order.request_date)
                            or seg_spm.specimen_received_date
                            or seg_obr.start_date
                            or message.message_date
                        ),
                        city=seg_pid.city_address,
                        postal_code=seg_pid.zip_address,
                        country=seg_pid.country_address,
                        address="~".join(
                            [
                                li
                                for li in [
                                    seg_pid.street_address,
                                    seg_pid.other_street_address,
                                ]
                                if li
                            ]
                        ),
                        phone=seg_pid.phone_number,
                    )
                    if seg_obr.relevant_clinical_info:
                        sample.diagnostic = seg_obr.relevant_clinical_info
                    if common_order:
                        for info in common_order.infos:
                            if info.comment:
                                sample.comments.get_or_create(
                                    type="sample", text=info.comment
                                )
                    for info in seg_obr.infos:
                        if info.comment:
                            sample.comments.get_or_create(
                                type="sample", text=info.comment
                            )
                    for info in seg_pid.infos:
                        if info.comment:
                            sample.comments.get_or_create(
                                type="patient", text=info.comment
                            )
                    sample.orders.get_or_create(
                        code=seg_obr.test_code, priority=seg_obr.priority, action=action
                    )
                    if seg_obr.ordering_service_code:
                        sample.services.get_or_create(
                            code=seg_obr.ordering_service_code,
                            name=seg_obr.ordering_service_name,
                            phone=seg_obr.telephone_doctor,
                        )
                    if (
                        seg_obr.code_doctor
                        and seg_obr.code_doctor != seg_obr.ordering_service_code
                    ):
                        doctor = sample.physicians.get_or_create(
                            code=seg_obr.code_doctor,
                            last_name=seg_obr.name_doctor,
                            type="prescriber",
                        )
                        if seg_obr.telephone_doctor:
                            doctor.phone = seg_obr.telephone_doctor
                    for seg_obx in seg_obr.results:
                        if not seg_obx.result_date and seg_obx.result_date_str:
                            logger.error(
                                "Résultat %s = %s, date erronée (%s) ignoré",
                                seg_obx.test_code,
                                seg_obx.observation_result_code,
                                seg_obx.result_date_str,
                            )
                            continue
                        if not seg_obx.result_date and seg_obx.test_code not in [
                            o.code for o in sample.orders
                        ]:
                            sample.orders.get_or_create(
                                code=seg_obx.test_code, action="add"
                            )
                        sample.results.get_or_create(
                            type=seg_obx.value_type,
                            anterior=True if seg_obx.result_date else False,
                            code=seg_obx.test_code,
                            value=seg_obx.observation_result_code,
                            date=seg_obx.result_date,
                        )
        return worklist
